import dynclass.BeanCreator;
import junit.framework.TestCase;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;

/**
 * Check basic operation of BeanCreator class.
 */
public class BeanTest extends TestCase {

    private HashMap hm;

    private Object nonPropertyKey = new Object();

    public BeanTest(String name) {

        super(name);
    }

    // try the read and write methods on the descriptor
    private void checkProperties(Object bean,
                                 PropertyDescriptor desc) throws Exception {

        String name = desc.getName();

        Method readMethod = desc.getReadMethod();
        assertNotNull("No read method for property: " + name, readMethod);

        Object value = readMethod.invoke(bean, null);
        assertEquals(hm.get(name), value);

        Method writeMethod = desc.getWriteMethod();
        assertNotNull("No write method for property: " + name, writeMethod);

        Object newValue = "newValueOf" + name;
        writeMethod.invoke(bean, new Object[] { newValue });
        value = readMethod.invoke(bean, null);
        assertEquals(newValue, value);
    }

    private void performTest(Object bean) throws Exception {

        BeanInfo bi = Introspector.getBeanInfo(bean.getClass());
        assertNotNull(bi);

        PropertyDescriptor[] pd = bi.getPropertyDescriptors();
        assertNotNull(pd);
        for (int i=0; i < pd.length; i++) {
            PropertyDescriptor desc = pd[i];
            if (hm.keySet().contains(desc.getName())) {
                checkProperties(bean, desc);

                // remove name - at end only non-property keys will remain
                hm.remove(desc.getName());
            }
        }
    }

    public void testGeneratedBean() throws Exception {

        hm = new HashMap();

        // the first two properties should be recognized by the
        // bean creator;  others will be ignored

        hm.put("goodProp1", new Integer(17));
        hm.put("URL", new Integer(96));
        hm.put("OddPropName", ""); // odd because initial capital is confusing
        hm.put(nonPropertyKey, "that key is not even a string");
        hm.put(this, this);

        performTest(BeanCreator.createBeanFromMap(hm));

        // check that unexpected keys did not become properties
        HashSet expected = new HashSet();
        expected.addAll(Arrays.asList(new Object[] { 
            nonPropertyKey,
            this
        }));

        assertEquals("Incorrect property set", expected, hm.keySet());
    }

    public void testCreateClass() throws Exception {

        String[] props = { "goodProp1", "URL" };
        Class beanClass = BeanCreator.createClassForProperties(props);

        for (int i=0; i < 2; i++) {
            hm = new HashMap();
            
            hm.put("goodProp1", new Integer(17));
            hm.put("URL", new Integer(96));
            
            performTest(BeanCreator.createBean(beanClass, hm));
            assertEquals(0, hm.size());
        }
    }
}
