import dynclass.ClassCreator;
import dynclass.ClassDefinition;
import dynclass.MethodDefinition;
import junit.framework.TestCase;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

public class ClassGenTests extends TestCase {

    static final InvocationHandler NULL_HANDLER = new InvocationHandler() {

            public Object invoke(Object proxy, Method m, Object[] args) {
                return null;
            }
        };

    public interface IntFn {

        public int fn(Integer arg);
        public int fni(float arg);
        public int fnfff(float a1, float a2, float a3);
        public int dMethod(double a1, double a2, double a3);
    }

    public ClassGenTests(String name) {

        super(name);
    }

    public void testCtor() throws Exception {

        ClassDefinition desc = new ClassDefinition(Object.class,
                                                   new Class[] { Cloneable.class },
                                                   new MethodDefinition[0]);

        Class cls = ClassCreator.defineClass(desc);

        Object obj = ClassCreator.newInstance(cls, NULL_HANDLER);
        assertTrue(obj instanceof Cloneable);
    }

    private MethodDefinition[] makeDescs(String[] names,
                                         Class[] rtypes,
                                         Class[][] params,
                                         Class[][] exceptions) {

        MethodDefinition[] descs = new MethodDefinition[names.length];
        for (int i=0; i < rtypes.length; i++) {
            descs[i] = new MethodDefinition(names[i],
                                            rtypes[i],
                                            params[i],
                                            exceptions[i]);
                                             
        }
        return descs;
    }

    public void testMethodHandlerInvocation() throws Exception {

        final Object[] instance = { null };
        final int[] count = { 0 };

        InvocationHandler h = new InvocationHandler() {
                public Object invoke(Object proxy, Method m, Object[] args) {
                    assertSame("Incorrect proxy", instance[0], proxy);
                    count[0]++;
                    return m.getName() + ":" + args.length;
                }
            };
        ClassDefinition desc = 
            new ClassDefinition(Object.class,
                                new Class[0],
                                makeDescs(new String[] { "fact", "noop" },
                                          new Class[] { String.class, Void.TYPE },
                                          new Class[][] {{ Integer.class, Float.class }, {}},
                                          new Class[][] {{}, {}}));

        Class cls = ClassCreator.defineClass(desc);

        Object obj = ClassCreator.newInstance(cls, h);
        instance[0] = obj;
        Method[] methods = cls.getDeclaredMethods();
        assertEquals(2, methods.length);

        Method fact = cls.getMethod("fact", new Class[] { Integer.class, Float.class });
        assertEquals(2, fact.getParameterTypes().length);
        String result = (String) methods[0].invoke(obj, new Object[] {
            new Integer(7),
            new Float(Math.PI),
        });
        assertEquals(result, "fact:2");

        Method noop = cls.getMethod("noop", new Class[0]);
        noop.invoke(obj, new Object[0]);
        noop.invoke(obj, new Object[0]);

        assertEquals(3, count[0]);
    }

    public void testPrimitiveReturns() throws Exception {

        InvocationHandler h = new InvocationHandler() {
                public Object invoke(Object proxy, Method m, Object[] args) {
                    
                    int result = 0;
                    for (int i=0; i < args.length; i++) {
                        result += ((Number)args[i]).intValue();
                    }
                    return new Integer(result);
                }
            };
        ClassDefinition desc = 
            new ClassDefinition(Object.class,
                                new Class[] { IntFn.class },
                                makeDescs(new String[] { "fn", "fni", "fnfff" },
                                          new Class[] { Integer.TYPE, Integer.TYPE, Integer.TYPE },
                                          new Class[][] {{ Integer.class }, { Float.TYPE }, { Float.TYPE, Float.TYPE, Float.TYPE } },
                                          new Class[][] {{}, {}, {}}));

        Class cls = ClassCreator.defineClass(desc);
        Object obj = ClassCreator.newInstance(cls, h);
        IntFn fobj = (IntFn) obj;
        int result = fobj.fn(new Integer(-42));
        assertEquals(-42, result);

        result = fobj.fni(12);
        assertEquals(12, result);

        result = fobj.fnfff(2.0f, 2.0f, 64.0f);
        assertEquals(68, result);

        assertEquals(4, fobj.dMethod(2.0, 1.0, 1.0));
    }
}
