import dynclass.ClassCreator;
import dynclass.ClassDefinition;
import dynclass.MethodDefinition;
import junit.framework.TestCase;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Ensure that abstract methods (and methods declared on interfaces)
 * are implemented on the generated class.
 */
public class ImpliedMethodTest extends TestCase {

    public static abstract class AnAbstractClass {

        public abstract int anAbstractMethod(String arg);

        public int aConcreteMethod() {

            return 0;
        }
    }

    public ImpliedMethodTest(String name) {

        super(name);
    }

    private Set toMethodDefSet(Method[] methods) {

        HashSet set = new HashSet();
        for (int i=0; i < methods.length; i++) {
            set.add(new MethodDefinition(methods[i]));
        }
        return set;
    }

    public void testImpliedMethods() throws Exception {

        Class[] infs = { Iterator.class };
        ClassDefinition def = new ClassDefinition(AnAbstractClass.class,
                                                  infs,
                                                  null);

        Class cls = ClassCreator.defineClass(def);

        Set declaredSet = toMethodDefSet(cls.getDeclaredMethods());
        Set iterSet = toMethodDefSet(Iterator.class.getMethods());
        
        if (!declaredSet.containsAll(iterSet)) {
            iterSet.removeAll(declaredSet);
            fail("Interface methods not declared: " + iterSet);
        }

        try {
            Method m = cls.getMethod("anAbstractMethod",
                                     new Class[] { String.class });
            assertEquals(cls, m.getDeclaringClass());
        }
        catch(NoSuchMethodException nsme) {
            fail("abstract method not declared");
        }

        // make sure this guy can be instantiated
        Object obj = ClassCreator.newInstance(cls, ClassGenTests.NULL_HANDLER);
        assertNotNull(obj.toString());
    }
}
