import dynclass.BeanCreator;
import junit.framework.TestCase;

import java.util.HashMap;
import java.util.HashSet;

/**
 * Ensure that ClassCreator / BeanCreator do not cause OutOfMemory
 * exceptions by growing without bound.
 */
public class MemoryTest extends TestCase {

    private long nextVal = 1;
    private static final int MAP_SIZE = 1000;

    public MemoryTest(String name) {

        super(name);
    }

    /**
     * Generate a string that starts with "prop" and is a 
     * JavaBeans-line property name.
     */
    private final String propString() {

        return "prop" + Long.toHexString(nextVal++);
    }

    private Object generateBeanFromMap() throws Exception {

        Object value = new Object();
        HashMap hm = new HashMap();

        for (int i=0; i < MAP_SIZE; i++) {
            hm.put(propString(), value);
        }
        return BeanCreator.createBeanFromMap(hm);
    }

    public void testMemory() throws Exception {

        HashSet accumSet = new HashSet();
        int iterations = 0;
        try {
            while (true) {
                Object bean = generateBeanFromMap();
                if (++iterations > 200) {
                    fail("Too much memory for this test - " + 
                         "try reducing the max memory by %33.");
                }
                // put classloader into set to prevent GC of class memory
                accumSet.add(bean.getClass().getClassLoader());
            }
        }
        catch(OutOfMemoryError oome) {
            if (iterations < 10) {
                fail("Not enough memory for testing - " + 
                     "increase max memory by 1MB");
            }
        }
        accumSet = null;

        // Now we've released the memory, so we should be able to 
        // run continuously.  
        for (int i=0; i < iterations * 5; i++) {
            try {
                generateBeanFromMap();
            }
            catch(OutOfMemoryError oome) {
                fail("Ran out of memory on iteration " + i);
            }
        }
    }
}
